#include <iostream>
#include <map>
#include "Maze.h"

using namespace std;

void serialize_maze_graph(Cell *c, ofstream& f)
{
  c->m_saved = true; // the cell is serialized only once

  f << endl << *c; // writing a new line with the cell id..
  f << c->m_nb_neighb; // .. then the number of neighbours
  for(int i = 0 ; i < c->m_nb_neighb ; i++) // then the ids of the neighbours
    f << *c->m_neighb[i];

  // Writing then each neighbour that has not already been serialized
  for(int i = 0 ; i < c->m_nb_neighb ; i++)
  {
    if(!c->m_neighb[i]->m_saved)
      serialize_maze_graph(c->m_neighb[i], f); // recursive call
  }
}

void serialize_maze(const Maze& maze, const string& file_name)
{
  ofstream f(file_name);

  if(!f.is_open())
    cout << "Impossible d'ouvrir le fichier en écriture" << endl;

  else
  {
    f << "# Start: " << endl << *maze.c0 << endl;
    f << "# End:  " << endl << *maze.cf << endl;
    f << "# Cells:";
    serialize_maze_graph(maze.c0, f); // writing the graph of cells
  }

  f.close();
}

Maze deserialize_maze(const string& file_name)
{
  Maze maze; // empty maze
  ifstream f(file_name);

  if(!f.is_open())
    cout << "Impossible d'ouvrir le fichier en lecture" << endl;

  else
  {
    string s;
    // The following is a temporary map of references,
    // it will not be used outside this block.
    map<pair<int,int>,Cell*> m_cells;

    getline(f, s); // reading text "# Start"

    Cell c0(0,0);
    f >> c0; // getting positions of the starting cell
    pair<int,int> p_c0 = {c0.m_x,c0.m_y}; // making a key reference
    f.get(); // so that we read the end of line

    getline(f, s); // reading text "# End"
    
    Cell cf(0,0);
    f >> cf; // getting positions of the exit cell
    pair<int,int> p_cf = {cf.m_x,cf.m_y}; // making a key reference
    f.get(); // so that we read the end of the line

    getline(f, s); // text "# Cells"

    while(!f.eof()) // eof: end of file
    {
      Cell *c = new Cell(0,0);
      f >> *c;

      if(f.eof()) // if end of file reached at this position...
      {
        delete c;
        break; // ...we break the while
      }

      m_cells[{c->m_x,c->m_y}] = c;

      int nb_neighb;
      f >> nb_neighb; // number of neighbours
      for(int i = 0 ; i < nb_neighb ; i++)
      {
        Cell c_n(0,0);
        f >> c_n;
        pair<int,int> p_n = {c_n.m_x,c_n.m_y};

        // If the neighbour already exists in the table:
        if(m_cells.find(p_n) != m_cells.end())
          c->add_neighb(m_cells[p_n]); // we add the one of the table as neighbour
      }

      f.get(); // so that we read the end of the previous line
    }

    maze.c0 = m_cells[p_c0];
    maze.cf = m_cells[p_cf];
  }

  f.close();
  return maze;
}