#include <iostream>

#include "Maze.h"
#include "maze_examples.h"
#include "Path.h"

#include "vibes.h"

using namespace std;

// (this method is already given)
void draw_gate(const Cell *n1, const Cell *n2)
{
  vibes::drawBox(min(n1->m_x, n2->m_x) + 0.1, max(n1->m_x, n2->m_x) + 0.9,
                 min(n1->m_y, n2->m_y) + 0.1, max(n1->m_y, n2->m_y) + 0.9,
                 "lightGray[lightGray]");
}

// (this method is already given)
void display_graph(Cell *cell)
{
  vibes::drawBox(cell->m_x, cell->m_x + 1, cell->m_y, cell->m_y + 1, "[lightGray]");
  cell->m_displayed = true;

  for(int i = 0 ; i < cell->m_nb_neighb ; i++)
  {
    if(!cell->m_neighb[i]->m_displayed)
      display_graph(cell->m_neighb[i]);
    draw_gate(cell, cell->m_neighb[i]);
  }
}

void display_maze(const Maze& maze)
{
  display_graph(maze.c0);
  vibes::drawCircle(maze.c0->m_x + 0.5, maze.c0->m_y + 0.5, 0.3, "#00AC07[#00AC07]");
  vibes::drawCircle(maze.cf->m_x + 0.5, maze.cf->m_y + 0.5, 0.3, "#F39F00[#F39F00]");
}

void display_path(const Path& path)
{
  if(path.m_next != nullptr)
  {
    vibes::drawBox(min(path.m_c->m_x, path.m_next->m_c->m_x) + 0.45, max(path.m_c->m_x, path.m_next->m_c->m_x) + 0.55,
                   min(path.m_c->m_y, path.m_next->m_c->m_y) + 0.45, max(path.m_c->m_y, path.m_next->m_c->m_y) + 0.55,
                   "#0088AC[#0088AC]");
    display_path(*path.m_next);
  }
}

bool find_path(Cell *c, Cell *cf, Path& path) // recursive way to find the path to the exit
{
  c->m_pathed = true;

  if(c == cf) // if the exit is the current cell
    return true;

  // Else, we test each neighbour
  for(int i = 0 ; i < c->m_nb_neighb ; i++)
  {
    if(c->m_neighb[i]->m_pathed)
      continue; // if this neighbour has already been analyzed, we test the next one

    // If this neighbour leads to the solution, we add it to the path
    if(find_path(c->m_neighb[i], cf, path))
    {
      path.add_to_path(c);
      return true;
    }
  }

  return false; // we did not find the path
}

int main()
{
  // If the executable creates its own maze:
  //  Maze maze = create_4x4_maze();
  //  serialize_maze(maze, "laby4x4.txt");

  // If the executable reads a maze from a file:
    Maze maze = deserialize_maze("laby10x10.txt");
  
  Path path(maze.cf); // the path to maze's exit (not computed yet)
  find_path(maze.c0, maze.cf, path);

  vibes::beginDrawing();
  vibes::newFigure("Maze");
  vibes::setFigureProperties("Maze", vibesParams("x", 200, "y", 200, "width", 700, "height", 700));
  vibes::axisLimits(0-0.5, 10+0.5, 0-0.5, 10+0.5);

  display_maze(maze);
  display_path(path);

  vibes::endDrawing();

  return EXIT_SUCCESS;
}